# Trims NEC-4 output file <filename>.NOU, sorting the output into the
# following ASCII text files:
#    <filename>.CM -- contains comment cards.
#    <filename>.AEG -- azimuth, elevation, and total gain table.
#    <filename>.CON -- drive conditions
#    <filename>.DAT -- gain statistics.
# 
# Call:
#    gawk -f noutrim.awk <target.xy> <filename.NOU>
#
# The target zones used to calculate statistics are defined by the file target.xy
# This file has two lines each containing 4 integers and a word:
#    Target 1: az-left to az-right, el-lo to el-hi, name (one word)
#    Target 2: az-left to az-right, el-lo to el-hi, name (one word)
# If a target zone is not needed, set the azimuths or elevations equal to each other.
# Both lines are REQUIRED, even if target zone(s) are not needed.
#
# NOTE: Points in Target zone 1 are EXCLUDED from the statistics for Target zone 2.
#
# NOTE: The variable LowestGainForAveraging is set to the lowest gain that will be used in averaging.
# This recognizes that real-world construction can not maintain tolerances and stability necessary for very low gain levels,
# e.g., more than 30 dB below the main beam of a large yagi.  Environmental scattering will also fill in deep nulls in patterns.
#
# There are several states in processing a .NOU file:
#    Init -- First pass through NEC output file; used to initialize noutrim's output files.
#    FindVersion -- Looking for NEC version type.
#    PreComment -- Skipping lines to reach the comment cards.
#    Comment -- Copying comment cards to the .CM file.
#    PreAEGHeader -- Searching for the start of az-el gain data table.
#    PreAEGData -- Found the header; skipping lines to start the data.
#    AEGData -- Now copying the data.
# Finally, the END routine writes out the accumulated statistics.
#
# 02 Sep 23: Added logic to process NEC-2D as well as NEC-4.1 files.
# 02 Sep 23: Located NEC version number.  Annotate comment file with NEC version.
# 02 Sep 23: Find comments correctly for NEC-4.1 files.  Still not correct for NEC-2D.  No Comment case also not correct.
# 02 Dec 30: v2.0 - Corrected drive impedance and current calculations for a current source.
# 02 Dec 30: v2.0 - Annotated comments with NouTrim version number.
# 03 Jan 24: v2.1 - Added error warning if azimuth data wraps past 360 degrees.
# 03 May 2:  v2.2 - Prints version number on screen.
# 03 Jul 1:  v2.3 - Corrected deviation calculations to use deviation from mean power.
# 03 Jul 2:  v2.4 - Fixed all known errors in processing NEC-2D and -4.1 files:
#                   -- Handles blank comment cards in the middle of the CM deck properly (see bug 02 Sep 23).
#                   -- To save space in the plot annotation, multiple consecutive blank CM lines collapse into one blank line.
#                   -- Handles case of no comments correctly.
#                   -- Does not plot the word "NOUTrim" twice in succession in the left annotation column.
#
# ===================================================================================================================
BEGIN {
   NoutrimVersion = "NOUTrim v2.4 2003 Jul 2 Wed"
   print NoutrimVersion

   # Logical constants
   TRUE = 1
   FALSE = 0

   # First round through NOU file will be for initialization.
   state = "Init"

   # Initialize variables for statistics outputs
   Gain = 0.0
   Azimuth = 0.0
   Elevation = 0.0
   LowestGainForAveraging = -15.0
   LowestPowerForAveraging = 10 ^ ( LowestGainForAveraging / 10)   # -15 dBi is the lowest gain that will be averaged.
   MaxGain = -100.00
   NrDataPoints = 0.0
   Power = 0.0

   # Initialize tag number to avoid having string forced into it.
   TagNr = 0.0

   # Get the target zones
      # Initialize variables to avoid having strings stuffed into the numeric ones.
      EXISTS = 1
      LEFT = 2
      RIGHT = 3
      LOW = 4
      HIGH = 5
      INCLUDESNORTH = 6
      NRDATAPOINTS = 7
      NAME = 8
      SUMPOWERS = 9
      MAXDBI = 10
      MAXDBIAZ = 11
      MAXDBIEL = 12
      MINDBI = 13
      MINDBIAZ = 14
      MINDBIEL = 15

      # Initialize array for two target zones and a third, untargeted region.
      for (i=1; i<=3; i++)
      {
         Target[ i, EXISTS] = FALSE
         Target[ i, LEFT] = 0.0
         Target[ i, RIGHT] = 0.0
         Target[ i, LOW] = 0.0
         Target[ i, HIGH] = 0.0
         Target[ i, INCLUDESNORTH] = 0.0
         Target[ i, NRDATAPOINTS] = 0.0
         Target[ i, SUMPOWERS] = 0.0
         Target[ i, MAXDBI] = -100.00		# NEC never generates more than +99.99 dBi nor less than -99.99 dBi gain.
         Target[ i, MAXDBIAZ] = 0.0
         Target[ i, MAXDBIEL] = 0.0
         Target[ i, MINDBI] = 100.00		# NEC never generates more than +99.99 dBi nor less than -99.99 dBi gain.
         Target[ i, MINDBIAZ] = 0.0
         Target[ i, MINDBIEL] = 0.0
      }

      # Now get the zone boundaries from the first input file (target.xy).
      for (i=1; i<=2; i++)
      {
         getline
         left = $1
         right = $2
         low = $3
         high = $4	
         if ( (left != right) && (low != high) )
         {
            Target[ i, EXISTS] = TRUE
            Target[ i, LEFT] = left
            Target[ i, RIGHT] = right
            Target[ i, LOW] = low
            Target[ i, HIGH] = high
            Target[ i, NAME] = $5
            if ( left > right ) Target[ i, INCLUDESNORTH] = TRUE

            # Write a description of the target borders for GMT to plot.
            # Vertical lines are great circle lines that follow longitude (azimuth).
            printf "%5.1f %4.1f\n%5.1f %4.1f\n>\n", left, low, left, high >"targetvertical.xy"    # great circle route up left longitude.
            printf "%5.1f %4.1f\n%5.1f %4.1f\n>\n", right, low, right, high >"targetvertical.xy"    # great circle route up right longitude.

            # Horizontal lines are non-great circles lines that follow latitude (elevation).  A separate GMT psxy statement is needed for these.
            printf "%5.1f %4.1f\n%5.1f %4.1f\n>\n", left, high, right, high >"targethorizontal.xy"    # straight across the top.
            printf "%5.1f %4.1f\n%5.1f %4.1f\n>\n", left, low, right, low >"targethorizontal.xy"    # straight across the bottom.
         }
      }
   # All finished with target zone definitions.
   # Any extra lines in the target zone file may cause trouble with the start of NEC .NOU file processing.
}

# ===================================================================================================================
# Line-by-line NEC output file processing
# This part of a GAWK command program is looped through until the entire contents of all remaining files have been processed.
{
   # Initializing noutrim's output files.
   if (state == "Init")
   {
      # make filenames.
      split( FILENAME, file, ".")
      CommentFile = file[1] ".CM"
      ConditionsFile = file[1] ".CON"
      AEGFile = file[1] ".AEG"
      StatisticsFile = file[1] ".DAT"

      # Initialize the GMT paragraph format strings for pstext in output files.
      printf ">  +0.00 3.50 8 0 0 TL 9p 4.00 l\n" >CommentFile
      printf ">  +0.00 3.50 8 0 0 TL 9p 2.00 l\n" >ConditionsFile
      printf ">  +0.00 3.50 8 0 0 TL 9p 4.00 l\n" >StatisticsFile

      state="FindVersion"
   }

   # ===================================================================================================================
   # Find the NEC version used to produce these results.
   if (state == "FindVersion")
   {
      if( $0 ~ /NUMERICAL ELECTROMAGNETICS CODE/)
      {
         NECversion = "unknown"		# Zero is flag value indicating no NEC version was recognized.

         if ($0 ~ /NEC-2D/ )
         {
            NECversion = "2D"
            state="PreComment"   # Must now hunt down a line that says "COMMENTS"
         }

         if ($0 ~ /NEC-4.1/ )
         {
            NECversion = "4.1"
            for (i=1; i<=9; i++) getline	            # skip 9 lines to reach first comment line.
            state = "Comment"
         }

         if (NECversion == "unknown")
         {
            printf "NEC version not recognized!\n"
            exit   # bails out, executing the END statements.
         }
      }
   }

   # ===================================================================================================================
   # Search for the start of comment cards.
   if (state == "PreComment")
   {
      if (NECversion == "2D" &&  $0 ~ /COMMENTS/)
      {
         # skip two blank lines to reach the first comment.
         getline
         getline
         state = "Comment"
	 getline		# Now grab the first line with a comment.
      }
   }   

   # ===================================================================================================================
   # Copy the NEC comment cards to the comment file.
   if (state == "Comment")
   {
      NrConsecutiveBlankLines = 0.0		# Used to avoid printing multiple blank lines or lines with asterisks that NEC4 inserts.
      NoCommentCards = TRUE                     # Used to detect the no comment card case.
      while (state == "Comment")
      {
         if( $0 ~ /STRUCTURE SPECIFICATION/)	# STRUCTURE SPEC means there are no more comment cards.
         {
            state = "PreEXCard"
            break		# Escape this While loop now.
         }
	 # Eliminate multiple blank lines or lines with all asterisks.
         if ((NF == 0) || ($0 ~ /\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*/ ) )
         {
            NrConsecutiveBlankLines++
            if ( NoCommentCards == FALSE && NrConsecutiveBlankLines == 1) printf "\n\n" >CommentFile		# Print just one consecutive blank line if it's not the first comment card.
         }
         else
         {
            NrConsecutiveBlankLines = 0.0
            NoCommentCards = FALSE		# Found at least one non-blank comment card.

            # NEC comment lines come with trailing spaces, which must be eliminated to plot correctly in GMT.
            LastChar = length()		# Find the last character number.
            LastNonSpaceChar = LastChar	# Starting at the right end of the line...
            while ( substr( $0, LastNonSpaceChar, 1) == " ") LastNonSpaceChar--	# Work towards the left until a non-space is found.
            $0 = substr( $0, 1, LastNonSpaceChar)		# Now set $0 to just the substring from start to last non-space char.

            sub( /\040\040/, "")		# First two chars are always a space and can be eliminated.
            gsub( /\040/, "\\040")		# Substitute '\040' for all other spaces to keep GMT from deleting the starting spaces.
            gsub( /\#/, "\\#")		# Replace # with \# to keep AWK from deleting lines of text.

            printf "%s\n\n", $0 >CommentFile		# Spit out the cleaned-up comment line.
         }
         getline		# Get the next line and see if it contains a comment.
      }
      # Seems like no more comments.
      if (NoCommentCards == TRUE) printf "(No comments.)\n\n" >CommentFile
      printf "NEC version %s\n", NECversion
      printf "NEC version %s\n\n", NECversion >CommentFile
      printf "\nK3NA %s\n\n", NoutrimVersion >CommentFile
   }

   # ===================================================================================================================
   # Searching for the quantity of EX cards, which will tell us how many feedpoints exist
   if (state == "PreEXCard")
   {
      if( NECversion == "4.1" && $0 ~ /INPUT LINE/)        # true when we hit the start of the input card listing.
      {
         NrEXCards = 0
         while ( $0 ~ /INPUT LINE/)
         {
            if ( $5 ~ /EX/) NrEXCards++
            getline    # grab the next line.  All the EX cards should be together.
         }
      }
      if( NECversion == "2D" && $0 ~ /DATA CARD/)        # true when we hit the start of the input card listing.
      {
         NrEXCards = 0
         while ( $0 ~ /DATA CARD/)
         {
            if ( $6 ~ /EX/) NrEXCards++
            getline    # grab the next line.  All the EX cards should be together.
         }
      }

      # No more INPUT LINE cards, so we are done counting EX cards.
      state = "PreFrequency"      
   }

   # ===================================================================================================================
   # Searching for the start of frequency data.
   if (state == "PreFrequency") {
      if ($0 ~ /FREQUENCY=/) {
         printf "%7.3f MHz\n\n\n", $2 >ConditionsFile
         state = "PreGround"
      }
   }

   # ===================================================================================================================
   # Searching for the start of ground information.
   if (state == "PreGround") {
      if ($0 ~ /FINITE GROUND/) {
         printf "%s\n\n", tolower( $0) >ConditionsFile
         getline
         printf "\\040\\040\\040%4.1f dielectric constant\n\n", $4  >ConditionsFile
         getline
         printf "\\040\\040\\040%6.1f mS/m\n\n\n", 1000 * $2  >ConditionsFile
         state = "PreImpedance"
      }
   }

   # ===================================================================================================================
   # Searching for drive point impedance results.
   if ( (state == "PreImpedance") && ($0 ~ /ANTENNA INPUT PARAMETERS/) ) {
      # true when we hit the top of the antenna input parameters listing.
      # The actual data is fixed field size.  Minus signs caused fields to run together, so normal GAWK processing can't be used.
      # Use substr to extract the correct data.

      printf "Drive impedance; voltage or current:\n\n" >ConditionsFile
      for (i=1; i<=3; i++) getline    # skip next three lines

      for (i=1; i<=NrEXCards; i++) {
         getline    # grab a line of data and extract values

         # For a voltage source model, assume the tag number is <900.
         # For a current source model, assume the tag number is >=900.
         # Multiply substr by 1.0 to force it to be a number.
         TagNr = 1.0 * substr( $0, 1, 6)

         if ( TagNr < 900) {
             Vr = substr( $0, 13, 12)
             Vi = substr( $0, 25, 12)
             Zr = substr( $0, 61, 12)
             Zi = substr( $0, 73, 12)
             Vmag = sqrt( Vr^2 + Vi^2)
             Vphase = (180/3.14159) * atan2( Vi, Vr)

             # special printing in order to put "j" and plus/minus signs in the right places
             printf "\\#%i:  ", i >ConditionsFile
             if (Zi > 0) printf "%7.1f + j%7.1f @~\\127@~", Zr,  Zi >ConditionsFile
             else        printf "%7.1f - j%7.1f @~\\127@~", Zr, -Zi >ConditionsFile
             printf "\\040\\040\\040%7.3fV @~\\320@~ %5.1f\\217\n\n", Vmag, Vphase >ConditionsFile
	 }
         else {
             # Current source model.  The following is per W4RNL email dated 02 Dec 24/0435 EST.
             # The listed "admittance" for the dummy wire is the actual drive impedance.
             # The listed "voltage" for the dummy wire is the actual drive current + 90 degrees.
             Ir = substr( $0, 13, 12)
             Ii = substr( $0, 25, 12)
             Zr = substr( $0, 85, 12)
             Zi = substr( $0, 97, 12)
             Imag = sqrt( Ir^2 + Ii^2)
             Iphase = -90 + ( (180/3.14159) * atan2( Ii, Ir))

             # special printing in order to put "j" and plus/minus signs in the right places
             printf "\\#%i:  ", i >ConditionsFile
             if (Zi > 0) printf "%7.1f + j%7.1f @~\\127@~", Zr,  Zi >ConditionsFile
             else        printf "%7.1f - j%7.1f @~\\127@~", Zr, -Zi >ConditionsFile
             printf "\\040\\040\\040%7.3fA @~\\320@~ %5.1f\\217\n\n", Imag, Iphase >ConditionsFile
         }
      }
      state = "PreAEGHeader"
    }

   # ===================================================================================================================
   # Searching for the start of az-el gain data.
   if ( (state == "PreAEGHeader") && ( $0 ~ /RADIATION PATTERNS/) )   # Found AEG header. Skip next 4 lines to reach data.
   {
      getline	# NEC 2D & 4.1:  This line is blank.
      getline	# NEC 2D & 4.1:  - - ANGLES - -
      getline	# NEC 2D & 4.1:  THETA     PHI
      getline	# NEC 2D & 4.1:  DEGREES  DEGREES
      state = "AEGData"
      getline	# First line of actual data.
   }

   # ===================================================================================================================
   # In AEG data until a blank record is reached.
   if (state == "AEGData") {
      if ( NF > 0) {
         # Convert elevation and azimuth information
         Azimuth = 360-$2
         Elevation = 90-$1
         if ( $2 > 360) {
            printf "Error: Az > 360 at phi=%5.2f, theta=%6.2f.\n", $2, $1
         }
         else {
            Gain = $5
            Power = 10^( Gain/10)
            if ( Power < LowestPowerForAveraging )
            {
               PowerOrFloor = LowestPowerForAveraging
               GainOrFloor = LowestGainForAveraging
            }
            else
            {
               PowerOrFloor = Power
               GainOrFloor = Gain
            }

            # Write to the Az-El-Gain file.
            printf "%6.2f %5.2f %6.2f\n", Azimuth, Elevation, Gain >AEGFile

            # Include for power average over whole hemisphere
            SumPower = SumPower + Power
            NrDataPoints++

            # Is this the largest gain encountered to date?
            if ( Gain > MaxGain)
            {
               MaxGain = Gain
               MaxGainAzimuth = Azimuth
               MaxGainElevation = Elevation
            }

            # Is this point within a target zone?
            WasInSomeTarget = FALSE   # Flag to check if point was included in a targeted zone.
            for( i=1; i<=2; i++)
            {
               InTarget = FALSE   # InTarget is forced to TRUE if the point is within Target zone 1 or 2.

               # Check for within range of elevations
               if ( (Target[ i, EXISTS] == TRUE) &&
                    ( Elevation >= Target[ i, LOW ] ) &&
                    ( Elevation <= Target[ i, HIGH] ) )
               {
                  # Check for within range of azimuths
                  if ( Target[ i, INCLUDESNORTH] == TRUE )
                  {
                     if ( ( Azimuth >= Target[ i, LEFT ] ) || ( Azimuth <= Target[ i, RIGHT] ) ) InTarget = TRUE
                  }
                  else
                  {
                     if ( ( Azimuth >= Target[ i, LEFT ] ) && ( Azimuth <= Target[ i, RIGHT] ) ) InTarget = TRUE
                  }
               }

               # If in target zone, compile statistics.
               if (InTarget == TRUE)
               {
                  # Remember that the point was in a targeted zone.
                  WasInSomeTarget = TRUE

                  Target[ i, NRDATAPOINTS]++
                  Target[ i, SUMPOWERS] = Target[ i, SUMPOWERS] + PowerOrFloor
                  TargetPower[ i, Target[ i, NRDATAPOINTS] ] = PowerOrFloor    # Array of powers maintained for each zone for calculating deviations.

                  if ( Gain > Target[ i, MAXDBI] )
                  {
                     Target[ i, MAXDBI] = Gain
                     Target[ i, MAXDBIAZ] = Azimuth
                     Target[ i, MAXDBIEL] = Elevation
                  }

                  if ( Gain < Target[ i, MINDBI] )
                  {
                     Target[ i, MINDBI] = Gain
                     Target[ i, MINDBIAZ] = Azimuth
                     Target[ i, MINDBIEL] =  Elevation
                  }

                  # Force an exit so that Target 1 data points are excluded from Target 2
                  i = 9999
               }
            }

            # If the point was not within either zone, add its statistics to the "untargeted" category.
            if ( WasInSomeTarget == FALSE)
            {
               Target[ 3, NRDATAPOINTS]++
               Target[ 3, SUMPOWERS] = Target[ 3, SUMPOWERS] + PowerOrFloor
               TargetPower[ 3, Target[ 3, NRDATAPOINTS] ] = PowerOrFloor	# Array of powers for calculating deviation.

               if ( Gain > Target[ 3, MAXDBI] )
               {
                  Target[ 3, MAXDBI] = Gain
                  Target[ 3, MAXDBIAZ] = Azimuth
                  Target[ 3, MAXDBIEL] = Elevation
               }

               if ( Gain < Target[ 3, MINDBI] )
               {
                  Target[ 3, MINDBI] = Gain
                  Target[ 3, MINDBIAZ] = Azimuth
                  Target[ 3, MINDBIEL] =  Elevation
               }
            }
         }
      }
      else state = "PreAEGHeader"
   }
}

# All done parsing the .NOU file.
END {
   print NrDataPoints " data points found."

   printf "%+4.2f dBi maximum gain at %3i az  %2i el.\n", MaxGain, MaxGainAzimuth, MaxGainElevation
   printf "Maximum gain:  %+4.2f dBi at %3i\217 az  %2i\217 el.\n\n", MaxGain, MaxGainAzimuth, MaxGainElevation >StatisticsFile

   printf "%5.1f %% power efficiency.\n", 100 * (SumPower/NrDataPoints) / 2
   printf "%5.1f %% power efficiency.\n\n", 100 * (SumPower/NrDataPoints) / 2 >StatisticsFile

   # Summarize the target zone statistics.
   for (i=1; i<=3; i++)
   {
      if ( (Target[ i, EXISTS] == TRUE) || (i == 3) )
      {
         AveragePower = Target[ i, SUMPOWERS] / Target[ i, NRDATAPOINTS]

         # Note: GAWK log function is base e.  Divide by 2.302585 to get base 10.
         AverageGain = 10 * log( AveragePower) / 2.302585

         # Calculate average deviation from mean gain
         # Note: the sum of deviations above a mean equals the sum of deviations below that mean.  However, the nr of points above and below a mean can be different.
         SumDeviation = 0.0
         NrPointsMinus = 0.0
         NrPointsPlus = 0.0
         for ( j=1; j<=Target[ i, NRDATAPOINTS]; j++)
         {
            Deviation = TargetPower[ i, j] - AveragePower
            if ( Deviation < 0)
	    {
                NrPointsMinus++
            }
            else
            {
                SumDeviation = SumDeviation + Deviation
                NrPointsPlus++
            }
         }
         AverageDeviationPlus = SumDeviation / NrPointsPlus
         AverageDeviationMinus = -SumDeviation / NrPointsMinus

         AverageDeviationPlusDB = 10 * log( (AveragePower + AverageDeviationPlus) / AveragePower) / 2.302585
         AverageDeviationMinusDB = 10 * log( (AveragePower + AverageDeviationMinus) / AveragePower) / 2.302585

         # Screen print
         printf "\nTarget %1i - %s: %3i-%3i az  %2i-%2i el.\n", i, Target[ i, NAME], Target[ i, LEFT], Target[ i, RIGHT], Target[ i, LOW], Target[ i, HIGH]
         printf "   %i data points.\n", Target[ i, NRDATAPOINTS]
         printf "   Mean gain*:  %+4.2f dBi.\n", AverageGain
         printf "   Avg pwr:   %8.6f\n", AveragePower
         printf "   Pwr dev:   %8.6f over %i points above and %i points below.\n", SumDeviation, NrPointsPlus, NrPointsMinus
         printf "   Avg dev from mean*: %+4.2f dB above, %+4.2f dB below.\n", AverageDeviationPlusDB, AverageDeviationMinusDB
         printf "   Max gain:  %+5.2f dBi at %3i az  %2i el.\n", Target[ i, MAXDBI], Target[ i, MAXDBIAZ], Target[ i, MAXDBIEL]
         printf "   Min gain:  %+5.2f dBi at %3i az  %2i el.\n", Target[ i, MINDBI], Target[ i, MINDBIAZ], Target[ i, MINDBIEL]

         # .DAT file print instructions for plotting
         if ( i == 3) printf "\nNon-targeted area statistics:\n\n" >StatisticsFile
         else printf "\nTarget %1i - %s:  %3i-%3i\217 az  %2i-%2i\217 el.\n\n", i, Target[ i,NAME], Target[ i,LEFT], Target[ i,RIGHT], Target[ i,LOW], Target[ i,HIGH] >StatisticsFile

         printf "\\040\\040\\040Mean gain*: %+4.2f dBi*.\n\n", AverageGain >StatisticsFile
         printf "\\040\\040\\040Average deviation above/below mean*: %+4.2f  %+4.2f dB\n\n", AverageDeviationPlusDB, AverageDeviationMinusDB >StatisticsFile
         printf "\\040\\040\\040Maximum gain: %+5.2f dBi at %3i\217 az  %2i\217 el.\n\n", Target[ i, MAXDBI], Target[ i, MAXDBIAZ], Target[ i, MAXDBIEL] >StatisticsFile
         printf "\\040\\040\\040Minimum gain: %+5.2f dBi at %3i\217 az  %2i\217 el.\n\n", Target[ i, MINDBI], Target[ i, MINDBIAZ], Target[ i, MINDBIEL] >StatisticsFile
      }
      else
      {
         printf "\nTarget %1i not used.\n", i
         printf "\n\nTarget %1i not used.\n\n", i >StatisticsFile
      }
   }

   # Footnote the minimum gain used in averaging within zones.
   printf "\n* Gain floor for calculating mean and deviation is %+3.1f dBi.\n", LowestGainForAveraging
   printf "\n* Gain floor for calculating mean and deviation is %+3.1f dBi.\n\n", LowestGainForAveraging >StatisticsFile
   printf "NEC x-axis is North azimuth.\n" >StatisticsFile

   close( CommentFile)
   close( ConditionsFile)
   close( AEGFile)
   close( StatisticsFile)
}
